part of 'zimkit_services.dart';

mixin ZIMKitMessageService {
  // Event
  Future<ZIMKitMessageListNotifier> getMessageListNotifier(String conversationID, ZIMConversationType type) {
    return ZIMKitCore.instance.getMessageListNotifier(conversationID, type);
  }

  ValueNotifier<ZIMKitReceivedMessages?> getOnMessageReceivedNotifier() {
    return ZIMKitCore().messageArrivedNotifier;
  }

  // API
  Future<int> loadMoreMessage(String conversationID, ZIMConversationType conversationType) async {
    return ZIMKitCore.instance.loadMoreMessage(conversationID, conversationType);
  }

  Future<void> sendTextMessage(
    String conversationID,
    ZIMConversationType type,
    String text, {
    FutureOr<ZIMKitMessage> Function(ZIMKitMessage)? preMessageSending,
    Function(ZIMKitMessage)? onMessageSent,
  }) async {
    return ZIMKitCore.instance.sendTextMessage(conversationID, type, text,
        preMessageSending: preMessageSending, onMessageSent: onMessageSent);
  }

  Future<void> sendFileMessage(
    String conversationID,
    ZIMConversationType type,
    List<PlatformFile> files, {
    bool autoDetectType = true,
    ZIMMediaUploadingProgress? mediaUploadingProgress,
    FutureOr<ZIMKitMessage> Function(ZIMKitMessage)? preMessageSending,
    Function(ZIMKitMessage)? onMessageSent,
  }) async {
    for (final file in files) {
      ZIMKitCore.instance.sendMediaMessage(conversationID, type, file.path!, ZIMMessageType.file,
          preMessageSending: preMessageSending, onMessageSent: onMessageSent);
    }
  }

  Future<void> sendMediaMessage(
    String conversationID,
    ZIMConversationType type,
    List<PlatformFile> files, {
    ZIMMediaUploadingProgress? mediaUploadingProgress,
    FutureOr<ZIMKitMessage> Function(ZIMKitMessage)? preMessageSending,
    Function(ZIMKitMessage)? onMessageSent,
  }) async {
    ZIMKitLogger.info('sendMediaMessage: ${DateTime.now().millisecondsSinceEpoch}');
    for (final file in files) {
      await ZIMKitCore.instance.sendMediaMessage(
          conversationID, type, file.path!, ZIMKit().getMessageTypeByFileExtension(file),
          preMessageSending: preMessageSending, onMessageSent: onMessageSent);
    }
    return;
  }

  Future<void> deleteMessage(List<ZIMKitMessage> messages) async {
    return ZIMKitCore.instance.deleteMessage(messages);
  }

  Future<void> recallMessage(ZIMKitMessage message) async {
    return ZIMKitCore.instance.recallMessage(message);
  }

  void downloadMediaFile(ZIMKitMessage message) {
    return ZIMKitCore.instance.downloadMediaFile(message);
  }
}
