// ignore_for_file: unused_field, avoid_print

import 'dart:convert';
import 'dart:math';

import 'package:crypto/crypto.dart';
import 'package:dio/dio.dart';
import 'package:faker/faker.dart';

var dio = Dio(BaseOptions(
  method: 'POST',
  headers: {'Content-Type': 'application/json'},
));

const int appid = 2031514356;
const String secret = '6e46fa37db195c449155c9fbfded9c4b';

Map<String, Object> generateSignature() {
  final timestamp = DateTime.now().millisecondsSinceEpoch ~/ 1000;
  final nonce = '${Random().nextInt(pow(2, 31).toInt())}';
  final commonParams = {
    'AppId': appid,
    'Timestamp': timestamp,
    'SignatureVersion': '2.0',
    'SignatureNonce': nonce,
  };

  final signature = md5.convert(const Utf8Encoder().convert('$appid$nonce$secret$timestamp'));
  commonParams['Signature'] = signature.toString();
  return commonParams;
}

const int startIndex = 0; // userID的起始值
const int batchSize = 60; // 注册几批 (每批100人)
const String userIDprefix = 'zimkit_';
Future<void> userRegister() async {
  for (var currentBatch = startIndex; currentBatch < startIndex + batchSize; currentBatch++) {
    final userList = List.generate(
      100,
      (index) {
        final userID = '$userIDprefix${currentBatch * 100 + index}';
        final userName = faker.person.name();
        return {'UserId': userID, 'UserName': userName, 'UserAvatar': 'https://robohash.org/$userID.png?set=set4'};
      },
    );

    final jsonData = jsonEncode({
      'UserInfo': [...userList]
    });
    print('--' * 100);
    print('currentBatch:$currentBatch');
    final response = await dio.request(
      'https://zim-api.zego.im',
      queryParameters: generateSignature()..['Action'] = 'UserRegister',
      data: jsonData,
    );
    print(response);
    await Future.delayed(const Duration(milliseconds: 60));
  }
}

const groupID = 'zimkit2';
Future<void> inviteUsersIntoGroup() async {
  for (var currentBatch = startIndex; currentBatch < startIndex + batchSize; currentBatch++) {
    final userList = List.generate(100, (index) => '$userIDprefix${currentBatch * 100 + index}');

    final jsonData = jsonEncode({'FromUserId': '446414120', 'GroupId': groupID, 'UserIds': userList});
    print('--' * 100);
    print('currentBatch:$currentBatch');
    final response = await dio.request(
      'https://zim-api.zego.im',
      queryParameters: generateSignature()..['Action'] = 'InviteUsersIntoGroup',
      data: jsonData,
    );
    print(response);
    await Future.delayed(const Duration(milliseconds: 1200));
  }
}

Future<void> removeUsers() async {
  // 一次50, 需要分批
  for (var i = 0; i < 2; i++) {
    for (var currentBatch = startIndex; currentBatch < startIndex + batchSize; currentBatch++) {
      final userList = List.generate(
        50,
        (index) => '$userIDprefix${currentBatch * 50 + index}',
      );

      final jsonData =
          jsonEncode({'FromUserId': '446414120', 'GroupId': groupID, 'CustomReason': 'reason', 'UserId': userList});
      print('--' * 100);
      print(jsonData);
      print('i:$i, currentBatch:$currentBatch');
      final response = await dio.request(
        'https://zim-api.zego.im',
        queryParameters: generateSignature()..['Action'] = 'KickoutGroupUser',
        data: jsonData,
      );
      print(response);

      await Future.delayed(const Duration(milliseconds: 60));
    }
  }
}

void main() async {
  // await userRegister();
  await inviteUsersIntoGroup();
  // await removeUsers();
}
